/**
 * 
 * @author anny
 *
 */
public class Avstandsberegn {
	
	private Kmeans k;
	private int nrOfClusters;
	private int x,y;
	private int[] klyngeTeller;
	private double[][] klyngeOversikt;
	
	/**
	 * Konstruerer et objekt av type Avstandsberegn til objektet av 
	 * type Kmeans. En matrise av type int oprettes for  holde styr 
	 * p antallet dokumenter i hver klynge.
	 * 
	 * @param k	objekt av type Kmeans.
	 */
	public Avstandsberegn(Kmeans k){
		this.k = k;
		nrOfClusters = k.getAntClusters();
		klyngeTeller = new int[nrOfClusters];
	}
	
	/**
	 * Metoden beregner x- og y-koordinater for komponenter ut fra bredden 
	 * og hyden i den aktuelle framen. Punktene blir beregnet slik at 
	 * komponentene skal ligge i sirkel rundt et senter (klynge).
	 * 
	 * @param i	angir hvor i sirkelen komponentet skal ligge
	 * @param b angir bredden til vinduet.
	 * @param h angir hyden til vinduet.
	 */
	public void beregnXogY(int i, int b, int h){
		double phi, theta, r;
		double klyngeNr = klyngeOversikt[i][0];
		double sim = klyngeOversikt[i][1];
		int antKlynger = k.getAntClusters();	
		
		if (!(klyngeNr == -1)){
			klyngeTeller[(int)klyngeNr]++;
		
			// y = r * sin theta 
			// theta = phi*i
			// phi = 2pi / ant. elem i en klynge
		
			phi = 2*Math.PI/(k.getClusterCounts()[(int)klyngeNr]);
			theta = phi*klyngeTeller[(int)klyngeNr];
			r = (1-sim)*(b/(antKlynger+1)/2);
			y = (int) (r * Math.sin(theta));
			x = (int) (r * Math.cos(theta));
				
			x = (int) (x + (klyngeNr+1)*(b/(nrOfClusters+1)));
			y = y + h/2;
		}
		else{
			x = -1;
			y = -1;
		}
	}
	
	/**
	 * Setter matrise som inneholder dokumentvektorene.
	 * 
	 * @param klyngeOversikt matrise som inneholder dokumentvektorene.
	 */
	public void setKlyngeOversikt(double[][] klyngeOversikt){
		this.klyngeOversikt = klyngeOversikt;
	}
	
	/**
	 * Setter alle verdier i matrisa klyngeTeller til 0.
	 *
	 */
	public void clearKlyngeTeller(){
		for (int i=0; i<nrOfClusters; i++){
			klyngeTeller[i] = 0;
		}
	}
	
	/**
	 * Returnerer verdien av x.
	 * 
	 * @return verdien av punkt x som int.
	 */
	public int getX(){
		return x;
	}
	
	/**
	 * Returnerer verdien av y.
	 * 
	 * @return verdien av punkt y som int.
	 */
	public int getY(){
		return y;
	}
}
