import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.HashMap;
import java.util.StringTokenizer;

/**
 * 
 * @author Anny Marit Svendsen
 *
 */
public class BuildIndex {
	private HashMap<String,IndexTermInfo> termIndex;
	
	/**
	 * Konstruerer objekt av type BuildIndex. Et nytt objekt av type
	 * HashMap blir opprettet for lagring av termIndex.
	 *
	 */
	public BuildIndex(){
		termIndex = new HashMap<String,IndexTermInfo>();
	}
	
	/**
	 * Leser fil og bygger opp indeks. Hver term blir kun lagt til
	 * en gang i indeksen. Indeksen bestr av termer og referanser
	 * til hvilke dokumenter de forekommer i.
	 * 
	 * @param inFile		filen som skal leses og indekseres.
	 * @param index			angir nummeret p filen.
	 * @param size			strrelsen p indeksen.
	 * @param isIndexFile	angir om det er en fil som skal indekseres 
	 * 						eller ikke. Det er kun filer som er valgt
	 * 						som vanlige dokumenter som skal indekseres.
	 * 						filer valgt som klyngerepresentanter skal
	 * 						ikke indekseres.
	 * @throws IOException
	 */
	public void makeIndex(File inFile, int index, int size, boolean isIndexFile) throws IOException{
		IndexTermInfo iTermInfo;
		
		if ((inFile.isFile()) && (inFile.exists())){
			FileReader fr = new FileReader(inFile);
			BufferedReader br = new BufferedReader(fr);
	
			String term = "";
			String textLine = br.readLine();
			StringTokenizer tok; 
			
			while (textLine != null){
				tok = new StringTokenizer(textLine," ");
				while (tok.hasMoreTokens()) {
					term = tok.nextToken();
					
					if (isIndexFile){
						if (termIndex.containsKey(term)){
							//termen finnes allerede i indeksen, s trenger bare  legge til dokumentnr
							//(hvis det ikke er gjort tidligere)
							iTermInfo = (IndexTermInfo)termIndex.get(term);
						}
						else{
							//termen eksisterer ikke i indeksen fra fr
							iTermInfo = new IndexTermInfo(size);
							termIndex.put(term, iTermInfo);
						}
						iTermInfo.setDok(index);
					}
					else{
						//det er ikke et av de ordinre dokumentene men en klyngerepresentant
						if (termIndex.containsKey(term)){
							iTermInfo = (IndexTermInfo)termIndex.get(term);
							iTermInfo.setDok(index);
						}
					}
				}
				textLine = br.readLine();
			}//end while
			
			br.close();
			fr.close();
		}
	}
	
	/**
	 * Returnerer hele hashmapet (indeksen) for dokumentene i samlingen.
	 * 
	 * @return hashmapet (indeksen) for dokumentene i samlingen.
	 */
	public HashMap<String,IndexTermInfo> getTermIndex(){
		return termIndex;
	}
}
