import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;

/**
 * 
 * @author Anny Marit Svendsen
 *
 */
public class Hierarkisk {

	private int antVectors;
	private Beregn beregn;
	private ArrayList<Klynge> klynger = new ArrayList<Klynge>();
	private File[] files;
	
	/**
	 * Konstuerer et objekt av type Hierarkisk med en matrise (double[][]).
	 * 
	 * @param vectors 	dokument-term-matrise. Inneholder alle 
	 * 					vektene til hver term i hvert dokument.
	 * @throws IOException 
	 */
	public Hierarkisk(double[][] vectors, File[] files) throws IOException{
		antVectors = vectors.length;
		this.files = files;
		System.out.println("antVectors : " + antVectors);
		for(int i=0; i<antVectors; i++){
			Klynge temp = new Klynge(i);
			Node tempNode = new Node(i, vectors[i]);
			temp.addNode(tempNode);
			temp.setMinimumDistanceNode(tempNode);
			klynger.add(temp);
		}
		beregn = new Beregn();
		/*System.out.println("starter med antall klynger : " + klynger.size());
		for(int bla=0; bla<klynger.size(); bla++){
			System.out.println("++++" + (Klynge)klynger.get(bla));
		}*/
		doClustering();
	}
	
	/**
	 * Utfrer hierarkisk klynging. Bruker en arrayliste med klynger 
	 * som inneholder egne lister med klynger som tilhrer klynga.
	 * @throws IOException 
	 *
	 */
	public void doClustering() throws IOException{
		int teller = 1;
		FileWriter fw = new FileWriter(new File("hierarkisk.inf"));
		BufferedWriter bw = new BufferedWriter(fw);
		while(klynger.size()>1){//itererer til man har alt i 1 klynge
		    for(int i=0; i<klynger.size(); i++){	    	
		        Klynge denneKlynge = (Klynge)klynger.get(i);
		        //initierer klyngeminimum p nytt
		        denneKlynge.setMinimumDistanceNode((Node)denneKlynge.getAllNodes().get(0));
		        
		        for(int j=i+1; j<klynger.size(); j++){
		            Klynge nesteKlynge = klynger.get(j);
		           
		            for(int k=0; k<denneKlynge.getAllNodes().size(); k++){
		            	Node denneNode = (Node)denneKlynge.getAllNodes().get(k);
			            
		            	for(int l=0; l<nesteKlynge.getAllNodes().size(); l++){	
			            	Node nesteNode = (Node)nesteKlynge.getAllNodes().get(l);
		                	double sim = beregn.sim_ij(denneNode.getVector(), nesteNode.getVector());
		                	//System.out.println("beregnet avstand : " + sim);
		                	if(denneNode.getDistance()<sim){//sim er ny avstand
		                		//bedre sim
		                		denneNode.setClosestKlynge(nesteKlynge);
		                		denneNode.setClosestNode(nesteNode);
		                		denneNode.setDistance(sim);
		                	}
		                }
		            }//endfor(k)
		        }//endfor(j)
		    }//endfor(i)
		    
		    double globaltBeste = ((Node)((Klynge)klynger.get(0)).getAllNodes().get(0)).getDistance();
		   // System.out.println("globalt beste sim initiert til " + globaltBeste);
		    
		    Klynge flytteKlynge = (Klynge)klynger.get(0);
		    Node flytteNode = (Node)flytteKlynge.getAllNodes().get(0);
		    for(int m=0; m<klynger.size(); m++){
		    	Klynge tempKlynge = (Klynge)(klynger.get(m));
		    	for(int n=0; n<tempKlynge.getAllNodes().size(); n++){
		    		Node tempNode = (Node)(tempKlynge.getAllNodes().get(n));
		    		if( tempNode.getDistance() > globaltBeste ){
		    			flytteKlynge = tempKlynge;
		    			flytteNode = tempNode;
		    		}
		    	}
		    }
		    
		    //flytt over alle nodene til closest klynge !
		    int tempItereringer = flytteKlynge.getAllNodes().size();
		    for(int o=0; o<tempItereringer; o++){
		    	Node no = (Node)flytteKlynge.getAllNodes().get(o); //exception
		    	flytteNode.getClosestKlynge().addNode(no);	
		    }
		    //sletter nodene i klyngen som ble flytta
		    flytteKlynge.clearNodes();
		    klynger.remove(flytteKlynge);
		    klynger.trimToSize();
		    
		    //skriver ut resultat til fil for hver iterasjon
		    //System.out.println("- iterasjon "+teller+" -");
		    try{
				bw.write("#### ITERASJON "+teller+" ####"+"\n");
			    for (int i=0; i<klynger.size(); i++){
			    	//System.out.println("klynge "+klynger.get(i).getId());
			    	//bw.write("-- KLYNGE "+klynger.get(i).getId()+"\n");
			    	bw.write("Klynge "+klynger.get(i).getId()+"\n");
			    	for (int j=0; j<klynger.get(i).getAllNodes().size(); j++){
			    		//System.out.println("      node "+((Node)klynger.get(i).getAllNodes().get(j)).getId());
			    		String filNavn = files[((Node)klynger.get(i).getAllNodes().get(j)).getId()].getName();
			    		filNavn = filNavn.substring(0, filNavn.length()-4);
			    		if ((j+1)%10==0 || (j+1)==klynger.get(i).getAllNodes().size()){
			    			//System.out.print(filNavn+", ");
			    			bw.write(filNavn+"\n");
			    		}
			    		else{
			    			//System.out.print(filNavn+"\n");
			    			bw.write(filNavn+", ");
			    		}
			    		((Node)klynger.get(i).getAllNodes().get(j)).setDistance(-1);
			    	}
			    	//System.out.println();
			    }//endfor(i)
			    bw.write("--------------------------\n");
		    }
		    catch(IOException e){};
		    //System.out.println("- - -");
		    teller++;
		}//endwhile
		System.out.println("hierarkisk analyse ferdig");
		bw.close();
		fw.close();
	}
}
