/*
 * Created on 13.mai.2005
 *
 */

/**
 * @author Per Kristen Fredlund
 *
 */
import java.io.Serializable;
import java.util.*;

public class IndexTermInfo implements Serializable
{
	private static final long serialVersionUID = 1L;
	private BitSet dokInfo;
	private int[] termCount;
	private double[] weights;
	private int uniqueDocs;
	private int maxFreq;
	private int mySize; //ant. dokumenter i indeksen (dokumenter+representanter)
	
	public IndexTermInfo(int size)
	{
		dokInfo = new BitSet(size);
		termCount = new int[size];
		weights = new double[size];
		uniqueDocs = 0;//number of docs that contain the term
		maxFreq = 0;//the highest occurence of the term in any doc
		mySize = size;
	}
	
	public int getUniqueDocs()
	{
		return uniqueDocs;
	}
	
	public void setWeights(Weights aWeight)
	{
		{
			for (int i=0; i<mySize; i++)
			{
				//calculate each weight here
				switch (aWeight)
				{
					case wBoolean : setBooleanWeights(); break;
					case wFrequency : setFrequencyWeights();break;
					case wFreqNorm : setFreqNormWeights(); break;
					case wTfIdf : setTfIdfWeights(); break;
					default : throw new AssertionError("Unknown weight-scheme");
				}
			}
		}
	}
	
	public void dumpWeights()
	{
		System.out.println("Dump:");
		for (int i=0; i<mySize; i++)
		{
			System.out.println(" " + weights[i]);
		}
		System.out.println("End Dump.");
	}
	
	private void setBooleanWeights()
	{
		//laterz ;-)
		for (int i=0; i<mySize; i++)
		{
			//set the weights
			weights[i]=getDokInt(i);
		}
	}
	
	private void setFrequencyWeights()
	{
		//laterz ;-)
		for (int i=0; i<mySize; i++)
		{
			//set the weights
			weights[i]=termCount[i];
		}
	}
	
	private void setFreqNormWeights()
	{
		//laterz ;-)
		for (int i=0; i<mySize; i++)
		{
			if (termCount[i] > 0)
			{	
				weights[i] = termCount[i];
				weights[i] = weights[i]/maxFreq;
			}
			else
			{
				weights[i] = 0;
			}
		}
	}
	
	private void setTfIdfWeights()
	{
		double temp;
		//laterz ;-)
		for (int i=0; i<mySize; i++)
		{
			temp = mySize;
			//System.out.println("Debug , temp=mySize = " + temp);
			temp = temp / uniqueDocs;
			//System.out.println("Debug , temp=temp/uniqueDocs = " + temp);
			
			if (termCount[i] > 0)
			{
				weights[i] = termCount[i] * Math.log(temp);
				//System.out.println("Debug , log= " + weights[i]);
			}
			else
			{
				weights[i] = 0;
			}
		}
	}
	
	public double getWeight(int pos)
	{
		if ((pos<mySize) && (pos>=0))
		{	
			return weights[pos];
		}
		else
		{
			return (double)0;
		}
	}
	
	public int getSize()
	{
		return mySize;
	}
	
	public int getMaxFreq()
	{
		return maxFreq;
	}
	
	public void setDok(int dokNr)
	{
		if (dokNr < mySize)
		{	
			if (! dokInfo.get(dokNr))
			{
				uniqueDocs++;
			}
			dokInfo.set(dokNr);
			termCount[dokNr]++;
			if (termCount[dokNr] > maxFreq){maxFreq = termCount[dokNr];}
		}
	}
	
	public boolean getDok(int dokNr)
	{
		if (dokNr < mySize)
		{	
			return dokInfo.get(dokNr);
		}
		else
		{
			return false;
		}
	}

	public int getDokInt(int dokNr)
	{
		int value=0;
		
		if (dokNr < mySize)
		{
			if (dokInfo.get(dokNr))
			{
				value = 1;
			}
			else
			{
				value = 0;
			}
		}
		return value;
	}
	
	public int getTermCount(int dokNr)
	{
		if (dokNr < mySize)
		{	
			return termCount[dokNr];
		}
		else
		{
			return 0;
		}	
	}
}
