import java.io.*;

/**
 * 
 * @author Anny Marit Svendsen
 *
 */
public class Kmeans {
	
	private double[][] vectors;
	private double[][] repVectors;
	private double[][] newRepVectors;
	private int[] clusterCount;
	private Beregn beregn;
	private int vectorLength;
	private int antClusters;
	private double[][] klyngeoversikt;
	private double threshold;
	private File[] files;
	
	/**
	 * Konstruerer et objekt av type Kmeans med matrisene for 
	 * dokumentvektorene og klyngerepresentantvektorene. Metoden
	 * doClustering() blir kalt opp.
	 * 
	 * @param vectors 		matrise som inneholder alle dokumentvektorene.
	 * @param repVectors 	matrise som inneholder alle klyngerepresentantvektorene.
	 * @param threshold		grenseverdi for tilordning til klynger.
	 */
	public Kmeans(double[][] vectors, double[][] repVectors, double threshold, File[] files){
		this.vectors = vectors;
		this.repVectors = repVectors;
		this.threshold = threshold;
		this.files = files;
		antClusters = repVectors.length;
		vectorLength = (vectors[0]).length;
		newRepVectors = new double[antClusters][];
		clusterCount = new int[antClusters];
		beregn = new Beregn();
		//matrise som holder styr p hvilken klynge vektorene hrer hjemme i
		klyngeoversikt = new double[vectors.length][2];	
		doClustering();
	}
	
	/**
	 * <b>K-means.</b>
	 * Alle dokumentvektorer blir sammenligned med hver klyngerepresentant 
	 * for  finne den som ligger nrmest. Dokumentvektorene blir tilordnet 
	 * den klyngen som har klyngerepresentant som ligger nrmest i forhold 
	 * til cosinusml.
	 * 
	 * Nye klyngerepresentanter blir s beregnet og overskrevet de gamle
	 * (matrisa repVectors).
	 * 
	 * Etter hver iterasjon i klyngefordelingen blir paint-metoden i klassen
	 * Painter kalt opp slik at den grafiske framstillingen kan oppdateres 
	 * underveis.
	 *
	 */
	private void doClustering(){
		boolean fortsette;
		double sim, bestSim = 0;
		int clusterNr = 0;
		
		Painter p = new Painter(this);
		p.GUI();
				
		do{
			fortsette = false;
			a_clear(clusterCount);
			bestSim = 0;
			
			for (int i=0; i<vectors.length; i++){
				boolean forkast = false;
				double[] vector = new double[vectorLength];
				System.arraycopy(vectors[i],0,vector,0,vectorLength);			
										
				//sammenligner dokumentvektor med hver klyngerepresentant for  
				//finne ut hvilken som er nrmest	
				for (int j=0; j<antClusters; j++){				
					sim = beregn.sim_ij(vector, repVectors[j]);
					if (sim>bestSim){
						bestSim = sim;
						clusterNr = j;
					}			
				}
				
				//------- legg inn en sjekk p om sim er under thresholdet --------------
				// da skal ikke vektoren vre i noen klynge, men forkastes
				if (bestSim < threshold){
					clusterNr = -1;
					bestSim = -1;
					System.out.println("objektet skal ikke vre med i noen klynger");
					forkast = true;
				}				
				System.out.println("vector "+i+" skal i klynge "+clusterNr+" sim: "+bestSim);
											
				klyngeoversikt[i][0] = clusterNr;
				klyngeoversikt[i][1] = bestSim;
				//ker antallet nr av vektorer i aktuell klynge 
				//(bare hvis den ikke skal forkastes)
				if (!forkast){
					clusterCount[clusterNr]++;
											
					double[] tempVector = new double[vectorLength];
					if (newRepVectors[clusterNr] != null){
						System.arraycopy(newRepVectors[clusterNr], 0,tempVector,0,vectorLength);
						
						//m oppdatere den bestemte vektoren (bestemte klynga)
						for (int j=0; j<tempVector.length; j++){
							tempVector[j] = tempVector[j] + vector[j];
						}
						newRepVectors[clusterNr] = tempVector;
					}
					else{
						newRepVectors[clusterNr] = vector;
					}
				}
				bestSim = 0;			
			}//endfor
						
			//beregne gjennomsnitt og sjekke om en skal avslutte klyngefordelingen
			for (int i=0; i<antClusters; i++){
				if (newRepVectors[i] != null){
					double[] tempVect = new double[vectorLength];
					System.arraycopy(newRepVectors[i],0,tempVect,0,vectorLength);			
					System.out.println("ant. i klynge "+i+": "+clusterCount[i]);
					
					//beregne gjennomsnitt
					for (int k=0; k<tempVect.length; k++){
						if (tempVect !=null ){
							tempVect[k] = tempVect[k]/clusterCount[i];
							//System.out.println("-- "+tempVect[k] +" "+clusterCount[i]);
						}
					}
					
					//sjekker om en skal avslutte klyngefordelingen
					sim = beregn.sim_ij(tempVect, repVectors[i]);
					if (sim == 1 || sim>0.9999){//ta hyde for at sim er nesten 1 (men ikke akkurat)
						//System.out.println(sim);
					}
					else{
						fortsette = true;
					}
					
					//setter nye klyngerepresentanter			
					repVectors[i] = tempVect;
				}
			}//endfor
				//slette innhold i newRepVectors
				for (int k=0; k<newRepVectors.length; k++){
					newRepVectors[k] = null;
				}
			
			
			/************* oppdaterer graf-framstilling ******************************/
			try {			
				p.update(p.getGraphics());
				Thread.sleep(1500);
			}
			catch(Exception e){}
			/*************************************************************************/
			
		}while(fortsette);
		
		for (int i=0; i< klyngeoversikt.length; i++){
			System.out.println("Dokument "+i+" skulle i klynge "+(int)klyngeoversikt[i][0]);
		}
		PaintKmeansResults finalPaint = new PaintKmeansResults(klyngeoversikt, antClusters);
		finalPaint.GUI();
		
		//resultatene skrives ut til fil
		SkrivTilFil skriv = new SkrivTilFil(klyngeoversikt, antClusters, files);
		try{	
			skriv.skriv();
			System.out.println("skriver til fil");
		}
		catch (IOException e){
			e.printStackTrace();
		}	
	}
	
	/**
	 * Returnerer matrise som inneholder vektorer for alle dokumenter.
	 * 
	 * @return verdien til matrisa av type double[][].
	 */
	public double[][] getKlyngeOversikt(){
		return klyngeoversikt;
	}
	
	/**
	 * Returnerer antall klynger.
	 * 
	 * @return verdien av denne int.
	 */
	public int getAntClusters(){
		return antClusters;
	}
	
	/**
	 * Returnerer antall dokumenter.
	 * 
	 * @return verdien av denne int.
	 */
	public int getAntDocs(){
		return vectorLength;
	}
	
	/**
	 * Returnerer matrisa som inneholder oversikt over antall
	 * dokumenter i hver klynge.
	 * 
	 * @return 	verdien av denne matrisa.
	 */
	public int[] getClusterCounts(){
		return clusterCount;
	}
	
	/**
	 * Setter hver verdi i matrisa til verdien 0.
	 * 
	 * @param a	matrise av type int[] som skal nullstilles.
	 */
	private void a_clear(int[] a){
		for (int i=0; i<a.length; i++){
			a[i] = 0;
		}
	}	
}
