
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.io.File;
import java.io.IOException;
import javax.swing.*;

/**
 * 
 * @author Anny Marit Svendsen
 *
 */
public class StartHierarchic extends JFrame implements ActionListener, ItemListener{
	
	private static final long serialVersionUID = 1L;
	private JButton openButton, startButton, exitButton;
	private JList fileList;
	private File[] files;
	private JComboBox weightTypeBox;
	private Weights weightType;
	
	/**
	 * Konstruerer et objekt av type JFrame.
	 * Kaller opp superklassen JFrame, setter strrelse p vindu 
	 * og lokasjon. Kaller opp gui().
	 *
	 */
	public StartHierarchic(){
		super("Agglomerativ hierarkisk klynging");
		setSize(700,500);
		setResizable(false);
		setLocationRelativeTo(null);
		weightType = Weights.wBoolean;
		gui();
		setVisible(true);
	}
	
	/**
	 * Tegner opp GUI for hierarkisk klynging.
	 *
	 */
	private void gui(){
		Color color = new Color(225,228,255);
		//henter ut panel fra framen
		JPanel panel = (JPanel)this.getContentPane();
		panel.setBorder(BorderFactory.createEmptyBorder(20,15,20,15));
		panel.setBackground(color);
		
		//################## NORTH PANEL #########################################
		JPanel northPanel = new JPanel(new GridLayout(2,1));
		northPanel.setBorder(BorderFactory.createEmptyBorder(0,0,5,0));
		northPanel.setBackground(color);
			
			//buttonpanel - knapper for  pne dokumenter (bde vanlige og klyngerepresentanter)
			JPanel buttonPanel = new JPanel(new GridLayout(1,6,5,5));
			buttonPanel.setBackground(color);
			buttonPanel.setBorder(BorderFactory.createEmptyBorder(5,0,0,0));
			buttonPanel.setBackground(color);
			openButton = new JButton("pne fil(er)...");
			openButton.addActionListener(this);
			buttonPanel.add(openButton);
			buttonPanel.add(new JLabel());
			buttonPanel.add(new JLabel());
			buttonPanel.add(new JLabel());
			buttonPanel.add(new JLabel());
			buttonPanel.add(new JLabel());
			
			JLabel info1 = new JLabel("Velg dokument du nsker  utfre klyngeanalyse p, " +
					"og evt. initielle klyngerepresentanter");
			northPanel.add(info1);
			northPanel.add(buttonPanel);
        
		//################## MAIN PANEL #########################################
		JPanel mainPanel = new JPanel(new GridLayout(1,1,3,0));
		mainPanel.setBackground(color);	
			
			//------- scroller for filListe - dokumenter ------------------------
			fileList = new JList();
			fileList.setFocusable(false);
			fileList.setSelectionBackground(Color.WHITE);
	        fileList.setSelectionForeground(Color.BLACK);
	        JScrollPane fileListScroller = new JScrollPane(fileList, 
	        		JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);   
	        fileListScroller.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(),
	        		"Dokumenter"));	
			fileListScroller.setBackground(color);
		
		mainPanel.add(fileListScroller);
		
        //################## SOUTH PANEL #########################################
		JPanel southPanel = new JPanel(new BorderLayout());
		southPanel.setBackground(color);
		southPanel.setBorder(BorderFactory.createEmptyBorder(10,0,0,0));			
			JPanel southButtons = new JPanel(new GridLayout(2,4,5,5));
			southButtons.setBackground(color);
			exitButton = new JButton("Avslutt");
			exitButton.addActionListener(this);
			exitButton.setForeground(Color.red);
			startButton = new JButton("Start");
			startButton.addActionListener(this);
			weightTypeBox = new JComboBox(new Weights[]{Weights.wBoolean, Weights.wFrequency});
			weightTypeBox.addItemListener(this);
			JLabel weightInfo = new JLabel("Velg vektingtype:");
			southButtons.add(weightInfo);
			southButtons.add(weightTypeBox);
			southButtons.add(new JLabel());
			southButtons.add(new JLabel());
			southButtons.add(new JLabel());
			southButtons.add(new JLabel());
			southButtons.add(new JLabel());
			southButtons.add(new JLabel());
			southButtons.add(new JLabel());
			southButtons.add(new JLabel());
			southButtons.add(startButton);
			southButtons.add(exitButton);
		southPanel.add(southButtons, BorderLayout.SOUTH);
		
		panel.add(northPanel, BorderLayout.NORTH);
		panel.add(mainPanel, BorderLayout.CENTER);
		panel.add(southPanel, BorderLayout.SOUTH);
	}
	
	/**
	 * Starter applikasjon for Hierarkisk klyngeanalyse.
	 * 
	 * @param args
	 */
	public static void main(String[] args) {
		try{
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		}
		catch (Exception e){
			System.out.println("setLookAndFeel didn't work! :)");
		}
		new StartHierarchic();

	}
	
	/**
	 * Lytter til filpnerknapp for dokumenter som skal klynges. 
	 * Lytter ogs til startknapp for start av klyngeanalyse 
	 * (bare hvis dokumenter er valgt), og avsluttknapp.
	 * 
	 *  (non-Javadoc)
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	public void actionPerformed(ActionEvent e) {
		if (e.getSource() == openButton){
			FilAapner fc    = new FilAapner();
	        files = fc.getFiles();
	        String currentDirectory = fc.getCurrentDirectory().getAbsolutePath();
	        if (files!=null){
		        System.out.println("current directory: "+currentDirectory);
		        System.out.println(currentDirectory);
		        fileList.setListData(files);
	        }
	        else{
	        	System.out.println("matrisa er tom");
	        }//endif
		}
		else if (e.getSource() == startButton){
			System.out.println("start clustering");
			try {
				if (files!= null){
					LagVektorer lag = new LagVektorer(files, weightType);
					double[][] vectors = lag.getVectors();
					new Hierarkisk(vectors, files);
				}
				else {
					JOptionPane.showMessageDialog(null, "You have to select documents to be clustered first", 
							"No documents selected",JOptionPane.INFORMATION_MESSAGE);
				}
			} catch (IOException e1) {
				e1.printStackTrace();
			}
		}
		else if (e.getSource() == exitButton){
			int exit = JOptionPane.showConfirmDialog(null, "Do you really want to Exit?", 
					"Confirm Exit", 0, JOptionPane.QUESTION_MESSAGE);
			if (exit==0){
				System.out.println("exit");
				System.exit(0);
			}
		}		
	}
	
	/**
	 * Lytter til vektingtypeboksen som angir hvilken vektingtype som skal benyttes.
	 * 
	 *  (non-Javadoc)
	 * @see java.awt.event.ItemListener#itemStateChanged(java.awt.event.ItemEvent)
	 */
	public void itemStateChanged(ItemEvent e) {
		Object source = e.getItemSelectable();
		if(source == weightTypeBox){
			System.out.println(((JComboBox) source).getSelectedItem());
			weightType = (Weights) (((JComboBox) source).getSelectedItem());
		}
		
	}
}
